package com.isu.gaswellwatch.modbus.data.decode.impl;

import com.isu.gaswellwatch.modbus.data.ModbusMessage;
import com.isu.gaswellwatch.modbus.data.decode.DecodeHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

/**
 * 本地日期时间类型解析器
 *
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 21:34
 */
@Component(LocalDateTimeDecodeHandler.NAME + DecodeHandler.DECODE_NAME)
public class LocalDateTimeDecodeHandler implements DecodeHandler {
    public static final String NAME = "fullYearLocalDateTime";

    public static final DateTimeFormatter IN_FORMATTER = DateTimeFormatter.ofPattern("yyyy-M-d H:m:s");
    public static final DateTimeFormatter OUT_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");


    @Override
    public String decode(Map<String, Object> commandPointMap, String value) {
        if (StringUtils.isBlank(value)) {
            return value;
        }
        return LocalDateTime.parse(value, IN_FORMATTER).format(OUT_FORMATTER);
    }

    @Override
    public void decode(Map<String, Object> commandPointMap, ModbusMessage.MessagePoint point) {
        point.setValue(this.decode(commandPointMap, point.getValue()));
    }

}
