package com.isu.gaswellwatch.vo.command.etc;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * 套压模式
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/12/11 9:22
 */
@Getter
@Setter
@SuperBuilder
public class CasPressureOptimization extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = -8310428465448926269L;

    /**
     * 套压开关
     * 261
     */
    private boolean casPressureSwitch;
    /**
     * 套压PSI比例
     * 262
     */
    private BigDecimal casPressureSensorRange;
    /**
     * 开井套压
     * 263
     */
    private BigDecimal openCasPressure;
    /**
     * 复位套压
     * 264
     */
    private BigDecimal openResetCasPressure;
    /**
     * 稳定时间
     * 265-267
     */
    private String openCasPressureStableTime;
    /**
     * 关井套压
     * 279
     */
    private BigDecimal closeCasPressure;
    /**
     * 跳闸套压
     * 280
     */
    private BigDecimal closeTripCasPressure;
    /**
     * 稳定时长
     * 281-283
     */
    private String closeCasPressureStableTime;

    public CasPressureOptimization() {
        this.setCode("ETC.CAS_PRESSURE_OPTIMIZATION");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && this.openCasPressure.compareTo(BigDecimal.ZERO) >= 0
                && this.closeCasPressure.compareTo(BigDecimal.ZERO) >= 0
                && StringUtils.isNotBlank(this.openCasPressureStableTime)
                && StringUtils.isNotBlank(this.closeCasPressureStableTime)
                && this.openResetCasPressure.compareTo(BigDecimal.ZERO) >= 0
                && this.closeTripCasPressure.compareTo(BigDecimal.ZERO) >= 0
                && this.casPressureSensorRange.compareTo(BigDecimal.ZERO) >= 0;
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        List<ModbusCommandDto> resultList = new ArrayList<ModbusCommandDto>(6);

        StringBuilder command = new StringBuilder(250);
        //               261
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0105    0007     0E
        command.append("0110010500070E");
        command.append("000").append(this.casPressureSwitch ? 1 : 0);
        command.append(Long.toHexString(this.casPressureSensorRange.longValue()));
        command.append(Long.toHexString(this.openCasPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(Long.toHexString(this.openResetCasPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(this.toHexString(this.openCasPressureStableTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());


        command.setLength(0);
        //               279
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0117    0005     0A
        command.append("0110011700050A");
        command.append(Long.toHexString(this.closeCasPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(Long.toHexString(this.closeTripCasPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(this.toHexString(this.closeCasPressureStableTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        return resultList;
    }

    public void setCloseCasPressureStableTime(String closeCasPressureStableTime) {
        this.closeCasPressureStableTime = this.timingValidate(closeCasPressureStableTime, "closeCasPressureStableTime");
    }

    public void setOpenCasPressureStableTime(String openCasPressureStableTime) {
        this.openCasPressureStableTime = this.timingValidate(openCasPressureStableTime, "openCasPressureStableTime");
    }

}
