package com.isu.gaswellwatch.vo.command.etc;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/12/11 9:22
 */
@Getter
@Setter
@SuperBuilder
public class TubPressureOptimization extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = 6405657928042198221L;

    /**
     * 套压开关
     */
    private boolean tubPressureSwitch;
    /**
     * 套压PSI比例
     */
    private BigDecimal tubPressureSensorRange;
    /**
     * 开井套压
     */
    private BigDecimal openTubPressure;
    /**
     * 复位套压
     */
    private BigDecimal openResetTubPressure;
    /**
     * 稳定时间
     */
    private String openTubPressureStableTime;

    public TubPressureOptimization() {
        this.setCode("ETC.TUB_PRESSURE_OPTIMIZATION");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && this.openTubPressure.compareTo(BigDecimal.ZERO) >= 0
                && StringUtils.isNotBlank(this.openTubPressureStableTime)
                && this.openResetTubPressure.compareTo(BigDecimal.ZERO) >= 0
                && this.tubPressureSensorRange.compareTo(BigDecimal.ZERO) >= 0;
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {

        StringBuilder command = new StringBuilder(250);
        //               331
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     014B    0007     0E
        command.append("0110014B00070E");
        command.append("000").append(this.tubPressureSwitch ? 1 : 0);
        command.append(Long.toHexString(this.tubPressureSensorRange.longValue()));
        command.append(Long.toHexString(this.openTubPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(Long.toHexString(this.openResetTubPressure.multiply(ONE_THOUSAND).longValue()));
        command.append(this.toHexString(this.openTubPressureStableTime));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setOpenTubPressureStableTime(String openTubPressureStableTime) {
        this.openTubPressureStableTime = this.timingValidate(openTubPressureStableTime, "openTubPressureStableTime");
    }

}
